/*!
 * openweather javascript library
 * http://www.spinetix.com/
 * Copyright 2013, SpinetiX S.A.
 * Released under the GPL Version 3 license.
 *
 * $Date: 2013-07-05 11:57:39 +0200 (Fri, 05 Jul 2013) $
 * $Revision: 19510 $
 */

/**
 * Messages that can be translated. 
 * @type {string}
 */
var message1 = "No data received for ";
var message2 = "City not found";

/**
 * Base source for the icons. 
 * This can be changed to a local path inside the project (e.g., "/icons").   
 * The default icon set can be downloaded and used locally, from http://download.spinetix.com/content/widgets/icons_weather.7z 
 * @type {string}
 */
var icon_src = "http://download.spinetix.com/content/widgets/icons/weather/";

/**
 * Checks the URI for additional options and adds them to the existing options.
 * @param {Object} options The object holding the widget options.
 */
function optionsFromURI( options ) {
    var uri = document.documentURI || window.location && window.location.href || document.location && document.location.href || 'http:///';
    query = $.decodeURIQueryString( uri );
    for ( var q in query ){
        if ( q in options ) {
            if ( options[q]===true || options[q]===false ){
                if ( query[q]=='no' || query[q]=='No' || query[q]=='false' || query[q]=='False' || query[q]=='off' || query[q]=='Off' )
                    options[q] = false;
                else
                    options[q] = query[q]?true:false;
            }else
                options[q] = query[q];
        }
    }
    
}

/**
 * Gets the image for a certain condition or a default one if not found.
 * For your own weather icon see: http://bugs.openweathermap.org/projects/api/wiki/Weather_Condition_Codes
 * @param {number} cond The weather condition as number.
 * @return {string} The path to the image.
 */
function getIcon( icon, weather_id ) {
    // return "http://openweathermap.org/img/w/" + icon + ".png";
    if ( icon === undefined ) {
        if ( weather_id !== undefined && weather_id >= 900  ) {
            return icon_src + weather_id + ".png";
        } else {
            return icon_src+"03d.png";
        }
    } else {
        return icon_src + icon + ".png";
    }
};

/**
 * Adds the city details into result.
 * @return {Object}
 */
function addCityInfo( json, result ) {
    if ( 'city' in json ) {
        result.name = json.city.name;
        result.id = "" + json.city.id;
        if ( 'coord' in json.city ) {
            result.lon = json.city.coord.lon;
            result.lat = json.city.coord.lat;
        }
        result.country = json.city.country;
    } else {
        if ( 'sys' in json ) {
            result.country = json.sys.country;
        }
        if ( 'coord' in json ) {
            result.lon = json.coord.lon;
            result.lat = json.coord.lat;
        }
        if ( 'name' in json ) {
            result.name = json.name;
        }
        if ( 'id' in json ) {
            result.id = "" + json.id;
        }
    }
    return result;
}

/**
 * Adds the weather details into result.
 * @return {Object}
 */
function addWeatherInfo( json, result ){
    if ( 'weather' in json ) {
        var obj = json.weather;
        if ( 'length' in obj && obj.length > 0 ){
            obj = obj[0];
        }
        result.weather = obj.main;
        result.description = obj.description;
        result.weather_id = obj.id;
        result.icon = obj.icon;
        result.icon_url = getIcon( result.icon, result.weather_id ); 
    }
    return result;
}

/**
 * Adds the wind details into result.
 * @return {Object}
 */
function addWindInfo( json, result ) {
    if ( 'wind' in json ) {
        result.wind_speed = json.wind.speed; 
        result.wind_deg = json.wind.deg; 
    } else {
        if ( 'speed' in json ) {
            result.wind_speed = json.speed;
        }
        if ( 'deg' in json ) {
            result.wind_deg = json.deg;
        }
    }
    return result;
}

/**
 * Adds the temperature details into result.
 * @return {Object}
 */
function addTemperatureInfo( json, result ) {
    if ( 'main' in json ) {
        result.temp = json.main.temp; 
        result.pressure = json.main.pressure;
        result.humidity = json.main.humidity;
        result.temp_min = json.main.temp_min;
        result.temp_max = json.main.temp_max;
    } else {
        if ( 'temp' in json ) {
            result.temp_min = json.temp.min;
            result.temp_max = json.temp.max;
            result.temp = json.temp.day;
            result.temp_day = json.temp.day;
            result.temp_night = json.temp.night;
            result.temp_eve = json.temp.eve;
            result.temp_morn = json.temp.morn;
        }
        if ( 'pressure' in json ) {
            result.pressure = json.pressure;
        }
        if ( 'humidity' in json ) {
            result.humidity = json.humidity;
        }
    }
    return result;
}

/**
 * Adds several details into result.
 * @return {Object}
 */
function addInfo( json, result ) {
    result.time = new Date(json.dt *1000 );
    addTemperatureInfo( json, result );
    addWindInfo( json, result );
    addWeatherInfo( json, result );
    return result;
}

/**
 * Parse a response from a call to current weather info:
 * http://api.openweathermap.org/data/2.5/weather?q=Lausanne&units=metric
 * @return {Object}
 */
function current( json ) {
    var result = {};
    if ( typeof json === 'string' ) {
        json = $.parseJSON( json );
    }
    // format the results into a single level json object
    result = addCityInfo( json, result );
    // system
    if ( 'sys' in json ) {
        result.sunrise = new Date( json.sys.sunrise * 1000 );
        result.sunset = new Date( json.sys.sunset * 1000 );
    }
    result = addInfo( json, result );
    return result;
}

/**
 * Parse a response from a call to current weather info for multiple cities. Note that City ID must be provided:
 *      http://api.openweathermap.org/data/2.5/group?id=524901,703448,2643743&units=metric
 * @return {Array}
 */
function group( json ) {
    if ( typeof json === 'string' ) {
        json = $.parseJSON( json );
    }
    var ret = [];
    for ( var i=0; i<json.list.length; i++ ) {
        ret.push(  current( json.list[i] ) );
    }
    return ret;
}

/**
 * Parse a daily forecast response
 *      http://api.openweathermap.org/data/2.5/forecast/daily?q=Lausanne&units=metric&cnt=3
 * Parse the hourly forecast for the next 5 days:
 *      http://api.openweathermap.org/data/2.5/forecast?q=Lausanne&units=metric
 * @return {Array}
 */
function forecast( json ) {
    var ret = [];
    if ( typeof json === 'string' ) {
        json = $.parseJSON( json );
    }
    for ( var i=0; i < json.list.length; i++ ) {		
        var result = {};
        addCityInfo( json, result );        // global info
        addInfo( json.list[i], result );    // forecast info
        ret.push( result );
    }
    return ret;
}

/**
 * Generic parsing function for all responses from http://api.openweathermap.org/data/2.5/
 * @return {Array}
 */
function parse( json ) {
    if ( json === "" ) {
        return [];
    }
    if ( typeof json === 'string' ) {
        json = $.parseJSON( json );
    }
    if ( 'cod' in json && json.cod == "404") {
        return [ { error: true, message: message2 } ];
    } else if ( 'list' in json ) {
        return ( 'city' in json ? forecast( json ) : group( json ) );
    } else {
        return [ current( json ) ];
    }   
}

/**
 * Generic function for the Open Weather Map API.
 * Requests the weather data from the API, parse the results and pass them to the user-defined callback function.
 * Possible end points:
 *   daily forecast: http://api.openweathermap.org/data/2.5/forecast/daily?q=Lausanne&units=metric&cnt=3
 *   hourly forecast: http://api.openweathermap.org/data/2.5/forecast?q=Lausanne&units=metric
 *   current weather: http://api.openweathermap.org/data/2.5/weather?q=Lausanne&units=metric
 *   current weather for multiple cities: http://api.openweathermap.org/data/2.5/group?id=524901,703448,2643743&units=metric
 * Note: this function is not used by the weather widgets, but was added for further extension of the widgets.
 * @param {Object} args The arguments passed by the caller.
 * @param {function(Array)} callback The function to be called after a successful request. The first parameter is the formatted 
 *        version of the data, whereas the second parameter corresponds to the raw data.
 */
function openweather( args, callback ) {
    /**
     * Defines various default settings.
     * @type {Object}
     */
    var settings = {
            apiURL: "http://api.openweathermap.org/data/2.5/",
            units: "metric", // set to "imperial" for fahrenheit
            location: '',
            lat: '', lon: '',
            id: '',
            days: 3, 
            what: 'current', // one of: daily, hourly, current
            refresh: 30*60*1000
    };
    args && $.extend(settings,  args);
    
    /**
     * Builds the URL for the request.
     * @return {string} The URL.
     */
    function requestURL( ) {
        var url = settings.apiURL;
        if ( settings.what == 'current' ){
            url += ( ( settings.id && settings.id.indexOf(",") != -1 ) ? "group?" : "weather?" );
        } else if ( settings.what=='daily' ){
            url += "forecast/daily?";
        } else {
            url += "forecast?";
        }
        if( settings.location !== '') {
            url += "q="+ encodeURIComponent(settings.location);
        } else if ( settings.id !== '' ) {
            url += "id="+settings.location;
        } else if ( settings.lat !== '' &&  settings.lon !== '' ) {
            url += "lat="+settings.lat+"&lon="+settings.lon;
        } else {
            throw ("Must specify one of the following: location, id or long+lat");
        }
        url += "&cnt=" + settings.days + "&units=" + settings.units;
        return url;
    }
    
    var key = "openweather:"+settings.what+":"+settings.location +":"+settings.id+":"+settings.lat+":"+settings.lon;
    function storeData( data ) {
        if ( data.length == 0) {
            if ( store !== null) {
                alert ("No data received. Using cached data from " + info.when);
                data = info.data;
            }
        } else {
            var now = new Date();
            var info = {
                date: now.toString(),
                data: data
            }
            localStorage.setItem( key,  JSON.stringify(info) );
        }
        callback( data );
    }
    
    function getData() {
        /**
         * Datasource configuration
         */
        var dataSource = {
            "parser": {
                "function": "parse",
                "type": "custom",
            },
            "type": "uri",
            "src": requestURL()
        }
        
        $.getData( dataSource, storeData );  
        $.setTimeout( getData, settings.refresh );   
    }
    
    // check the local storage
    var store = localStorage.getItem( key );
    alert( store );
    if ( store === null ) {
        getData();
    } else {
        var info = $.parseJSON( store );
        var now = new Date();
        var when = new Date( info.date );
        if ( now-when>settings.refresh || !('data' in info) )
            getData();
        else {
            $.setTimeout( getData, settings.refresh-(now-when) ); 
            callback( info.data );
        }
    }
    
    $.setInterval( getData, settings.refresh );   
}

/**
 * Weather widgets specific function for the Open Weather Map API.
 * Requests the weather data from http://openweathermap.org/, parse the results and pass them to the user defined callback function.
 * @param {Object} args The arguments passed by the caller.
 * @param {function(Array)} callback The function to be called after a successful request. The first parameter is the formatted 
 *        version of the data, whereas the second parameter corresponds to the raw data.
 */
function openweather_locations( args, callback ) {

    //-- variables section
    
    /**
     * Defines various default settings.
     * @type {Object}
     */
    var settings = {
            apiURL: "http://api.openweathermap.org/data/2.5/",
            units: "metric", // set to "imperial" for Fahrenheit
            locations: '',
            what: 'current', // one of: current, today, tomorrow, n (number of days in a 2d array)
            refresh: 30*60*1000, // refresh period in ms
            debug: false, // set to true to turn on debug alerts
            apiid: null // API ID
    };
    
    /**
     * The results to pass to the callback function.
     * @type {Array.<Object>}
     */
    var results = [];
    
    /**
     * Counter of results received.
     * @type {number}
     */
    var cnt = 0;
    
    //-- functions section
    
    /**
     * Builds the URL for the request.
     * @param {string} location 
     * @return {string} The request URL.
     */
    function requestURL( location ) {
        var url = settings.apiURL;
        if ( settings.what == 'current' ){
            url += "weather?";
        } else {
            var n = parseInt( settings.what, 10 );
            if ( isNaN(n) ) {
                n = 2;
            }
            url += "forecast/daily?cnt=" + n + "&";
        }
        url += "q=" + encodeURIComponent( location );
        url += "&units=" + settings.units;
        if ( settings.apiid != null ) {
            url += "&APPID=" + settings.apiid;
        }
        return url;
    }

    /**
     * Process.
     * @param {number} idx The location index. 
     * @param {string} location The location name.
     */
    function process( idx, location ) {        
        var key = "openweather:" + settings.what + ":" + location;
        var store = localStorage.getItem( key );
        var error_msg = message1 + location;
        
        /**
         * Sets the weather data of the location into the results array.
         * @param {Object} data The data returned by the API.
         * @param {boolean} formated Flag indicating if the data is already formatted.
         */
        function set_results( data, formated ){
            if ( data.length == 0 ) {
                // try to use the cache if no data is received
                if ( store !== null ) {
                    var info = $.parseJSON( store );
                    alert ( error_msg + " Using cached data from " + info.date);
                    results[idx] = info.data;
                } else {
                    alert ( error_msg );
                    results[idx] = { error: true, message: error_msg};
                }
            } else {
                // new data from the API
                if ( formated ) {
                    results[idx] = data;
                } else if ( data[0].error === true ) {
                    alert ( message2 );
                    results[idx] = data[0];
                } else {
                    // format the data
                    if ( settings.what == "tomorrow" ) {
                        results[idx] = data[1];
                    } else if ( settings.what == "current" || settings.what == "today" ) {
                        results[idx] = data[0];
                    } else {
                        results[idx] = {
                            name: data[0].name,
                            id: data[0].id,
                            lon: data[0].lon, 
                            lat: data[0].lat, 
                            country: data[0].country,
                            forecast: data
                        }; // keep the full data as well (this will create a 2d array).
                    }
                    
                    // store the data into cache
                    var now = new Date();
                    var info = {
                        date: now.toString(),
                        data: results[idx]
                    }
                    localStorage.setItem( key,  JSON.stringify(info) );
                }                        
            }
            cnt++;
            if ( cnt == settings.locations.length ) {
                // all the data has been received
                callback( results );
                cnt = 0;
            }
        }
        
        /**
         * Gets the weather data of the location from the API, parse it and pass it to set_results function.
         * The function calls itself after the refresh period.
         */
        function get_data() {
            if ( settings.debug ) {
                alert("Refreshing data for " + location);
            }
            var dataSource = {
                "parser": {
                    "function": "parse",
                    "type": "custom",
                },
                "type": "uri",
                "src": requestURL( location )
            }
            
            $.getData( dataSource, set_results );
            $.setTimeout( get_data, settings.refresh ); 
        }
        
        // let's do some processing...
        if ( store === null ) {
            get_data();
        } else {
            if ( settings.debug ) {
                alert( store );
            }
            var info = $.parseJSON( store );
            var now = new Date();
            var when = new Date( info.date );
            var t = settings.refresh - ( now - when );
            if ( t <= 0 || !('data' in info) ) {
                get_data();
            } else {
                $.setTimeout( get_data, t ); 
                set_results( info.data, true );
            }
        }
        // end of processing.
    }
    
    //-- main code section
    
    args && $.extend(settings,  args);
    if ( settings.locations && typeof( settings.locations ) == "string" ) {
        settings.locations = [settings.locations];
    }
    
    for ( var i=0; i < settings.locations.length; i++ ){
        process( i, settings.locations[i] );
    }
}